#ifndef _BLOB_TRACKER_H
#define _BLOB_TRACKER_H

#include "ofMain.h"
#include "ofAddons.h"

#include "appConstants.h"

/* //moved to appConstants.h
typedef struct{

	int id;

	ofVec2f p;
	ofVec2f v;
	
	int pArea;
	int bArea;
	int length;
	float density;
	
	int birthTime;
	float speed;
		
}cvData;
*/

#include "baseGui.h"


class blobTracker{
	
	public:
		blobTracker(){
		
		}

		~blobTracker(){
			delete [] prevBlobs;
		}


		void setup(int w, int h, int numBlobsToConsider){
			
			width 			= w;
			height 			= h;
			numBlobs 		= numBlobsToConsider;
			lastNumFound	= numBlobs;
			prevBlobs		= new cvData[numBlobs];
			genID			= 0;
			lastTime		= 0;
			
			memset(prevBlobs, 0, sizeof(cvData)*numBlobs);

		}
		
		void resetCounter(){
			genID = 0;
		}
		
		int getBlobID(int which){
			if(which >= lastNumFound) return -1;
			return prevBlobs[which].id;
		}
		
		int getBlobDuration(int which){
			if(which >= lastNumFound) return -1;
			return ofGetElapsedTimeMillis()-prevBlobs[which].birthTime;
		}
		
		float getBlobActivity(int which){
			if(which >= lastNumFound) return 0;
			return prevBlobs[which].speed;
		}
		
		cvData getDataStruct(int which){
			if(which >= lastNumFound) return cvData();
			return prevBlobs[which];
		}
		
		void identifyBlobs(ofCvBlob * blobs, int numFound, int maxDist){
		 	
		 	
		 	numFound = MIN(numBlobs, numFound);
		 	
		 	if(numFound == 0)return;
		 	
		 	cvData * tmpBlobs = new cvData[numFound];
		 	
		 	for(int i = 0; i < numFound; i++){
		 		tmpBlobs[i].p.set(blobs[i].centroid.x, blobs[i].centroid.y);
		 		tmpBlobs[i].pArea 	= blobs[i].area;
		 		tmpBlobs[i].bArea 	= blobs[i].boundingRect.width * blobs[i].boundingRect.height;
		 		tmpBlobs[i].length 	= blobs[i].length;
		 		tmpBlobs[i].density	= tmpBlobs[i].pArea/tmpBlobs[i].bArea;		 		
		 		
		 		float shortestDistance 	= 99999999;
		 		int tmpID 				= genID;
		 		int which 				= -1;
		 		bool found 				= false;
		 		float speed 			= 0.0;
		 		
		 		for(int j = 0; j < lastNumFound; j++){
		 		
		 			if(prevBlobs[j].id == -1) continue;
		 		
		 			ofVec2f dist(tmpBlobs[i].p.x - prevBlobs[j].p.x, tmpBlobs[i].p.y - prevBlobs[j].p.y);
		 			float len = dist.length();
		 			
		 			if(len < maxDist && len < shortestDistance){
		 				shortestDistance = len;
		 				tmpID = prevBlobs[j].id;	
		 				found = true;
		 				which = j;	
		 				speed = len;
		 			}
		 						 		
		 		}
		 				 		
		 		tmpBlobs[i].id 		= tmpID;
		 		
		 		if(found){
		 			prevBlobs[which].id = -1;
		 			tmpBlobs[i].birthTime = prevBlobs[which].birthTime;
		 			
		 			int timeElapsed = ofGetElapsedTimeMillis() - lastTime;
		 			float factor = 1.0;
		 			
		 			if(timeElapsed > 16){
		 				factor = ((float)timeElapsed / 16.667);	
		 				//printf("elapsed %i \n", timeElapsed);
		 			}
		 			
		 			float amnt = pow(0.95, factor);
		 			float add =  pow(0.05,  1.0/factor);
	
		 			tmpBlobs[i].speed = prevBlobs[which].speed * amnt + speed * add;
		 		}else{
		 			tmpBlobs[i].birthTime = ofGetElapsedTimeMillis();		 			
		 			//printf("new blob %f %f\n",blobs[i].centroid.x, blobs[i].centroid.y);
		 			genID++;
		 		}	
		 		
		 		lastTime = ofGetElapsedTimeMillis();		 				 		
		 	}
		 	
		 	for(int i = 0; i < numFound; i++){
				prevBlobs[i] = tmpBlobs[i];	
				
			}		 		
		 
		 	lastNumFound = numFound;	
		 	
		 	delete [] tmpBlobs;
		 		 	 	
		}		
		
		
	protected:
		int width;
		int height;
		int numBlobs;
		int lastNumFound;
		int genID;
		int lastTime;
		cvData * prevBlobs;
				
};

#endif