#ifndef _CV_MANAGER_H
#define _CV_MANAGER_H

#include "ofMain.h"
#include "ofAddons.h"

#include "baseGui.h"
#include "blobTracker.h"
#include "contourAnalyser.h"
#include "cornerFinder.h"

#define MAX_CONTOURS 10000
#define NUM_CARDS 9

static int pnpoly (int npol, float *xp, float *yp, float x, float y){
	int i, j, c = 0;
	for (i = 0, j = npol-1; i < npol; j = i++) {
	 if ((((yp[i]<=y) && (y < yp[j])) ||
	      ((yp[j]<=y) && (y < yp[i]))) &&
	       (x < (xp[j] - xp[i]) * (y - yp[i]) / (yp[j] - yp[i]) + xp[i]))
	   c = !c;
	}
	return c;
}



typedef struct{
	ofVec2f c[4];
	bool good;
	float pixelRatio;
	int whichCard;
	int pta;
	int ptb;
	int ptc;
	int ptd;
	float angle;
	float hu;
	ofVec2f center;
}card;



class cvManager : public baseGui{
	
	public:
		cvManager(){
			treeCandidateID = -1;
			newData 		= false;
			bSetup			= false;
			which			= 0;
			numPoints		= 0;
			
			memset(cardVal, 0, 10 * sizeof(float));
		}
		
		//changing cameras or switching from/to the camera mode
		//requires the app to be restarted - mabe we can change this?
		//---------------------------		
		void setupCamera(int deviceNumber, int width, int height){
			VG.setDeviceID(deviceNumber);
			VG.initGrabber(width, height);
			W = VG.width;
			H = VG.height;			
			bCameraSetup = true;
			which = 2;

			setupCV();
		}
		
		//same as above - from/to using test movies
		//requires a restart
		//---------------------------		
		void setupVideo(string videoPath){
			
			VP.loadMovie(videoPath);
			VP.play(); 
			VP.setPosition(0.0);
			VP.setSpeed(0.90);
			VP.setUseTexture(true);
			W = VP.width;
			H = VP.height;
			bVideoSetup = true;
			which = 1;
			
			setupCV();
		}
		
		//good for adjusting the color balance, brightness etc
		//---------------------------		
		void openCameraSettings(){
			if(bCameraSetup)VG.videoSettings();			
		}
		
		//---------------------------		
		void useCamera(){
			if(bCameraSetup)which = 2;	
		}		
		
		//---------------------------		
		void useVideo(){
			if(bVideoSetup)which = 1;	
		}	
		
		//set background 
		//---------------------------
		void captureBackgroundFrame(){
		
			unsigned char * pixCam;

			if(which == 1){
				VP.idleMovie();	
				pixCam 	= VP.getPixels();
			}
		    else if(which == 2 ){
				VG.grabFrame();
				pixCam 	= VG.getPixels();		
			}else{
				return;
			}
			
			VideoFrame.setFromPixels(pixCam, W, H);
			GreyFrame.setFromColorImage(VideoFrame);
			BackgroundFrame = GreyFrame;
		}
		
		
		//do all our openCV allocation
		//---------------------------		
		void setupCV(){
			
			if(bSetup) return;
			
			//our cardBank
			memset(cardBank, 0, W*H);	
				
			//our openCV inits
			VideoFrame.allocate(W, H);
			BankFrame.allocate(W, H);
			PresenceFrame.allocate(W, H);
			GreyFrame.allocate(W, H);
			HuFrame.allocate(64, 80);
			BackgroundFrame.allocate(W, H);
			WarpedFrame.allocate(W, H);
			CardWarpFrame.allocate(W,H);
			tmp.allocate(W, H, OF_IMAGE_GRAYSCALE);
			tmp.loadImage("bank.png");
			
			if(tmp.width > 0){
				unsigned char * p = tmp.getPixels();
				memcpy(cardBank, p, W*H);
				BankFrame.setFromPixels(p, W, H);
			}
			
			//this is so we can select a sub region of the 
			//camera image and warp it to full 320 by 240
			warpSrc[0].x = 0;
			warpSrc[0].y = 0;
			
			warpSrc[1].x = 1.0;
			warpSrc[1].y = 0;
			
			warpSrc[2].x = 1.0;
			warpSrc[2].y = 1.0;

			warpSrc[3].x = 0;
			warpSrc[3].y = 1.0;
			
			//Now we update out quad with the default
			//values and if then if the file is found
			//we overwrite with values from xml file
			QUAD.setup("QUAD_SRC_");
			QUAD.setQuadPoints(warpSrc);
			QUAD.readFromFile("settings/quad.xml");
			
			//warp dst will always be to 320 by 240
			//unless you want shit slow!
			warpDst[0].x = 0;
			warpDst[0].y = 0;
			
			warpDst[1].x = 320;
			warpDst[1].y = 0;
			
			warpDst[2].x = 320;
			warpDst[2].y = 240;

			warpDst[3].x = 0;
			warpDst[3].y = 240;	
			
			blobTrack.setup(W, H, 200);
			
			TYPE.loadFont("type/pala.ttf", 128);
			
			bSetup = true;
		}
		
		
		
		//-----------------------------
		void identifyShortestSide(card &theCard){
			int dist = 999999;
			int pta	 = 0;
			int ptb  = 0;
			
			for(int i = 0; i < 4; i++){
				for(int j = 0; j < i; j++){
					float d = sqrt( pow(theCard.c[i].x - theCard.c[j].x , 2) + pow(theCard.c[i].y - theCard.c[j].y , 2) );
				 	
				 	if(d < dist){
				 		pta = i;
				 		ptb = j;
				 		dist = d;
				 	}
				 }
			}
			
			//printf("pta is %i ptb is %i \n", pta, ptb);
			
			theCard.pta = pta;
			theCard.ptb = ptb;
			
		}
		
		//-----------------------------
		void calculateCenter(card &theCard){
			
			float avgX = 0;
			float avgY = 0;
			
			for(int i = 0; i < 4; i ++){
				avgX += theCard.c[i].x;
				avgY += theCard.c[i].y;				
			}
			
			avgX /= 4;
			avgY /= 4;
			
			theCard.center.x = avgX;
			theCard.center.y = avgY;
			
		}
		
		//-----------------------------
		void calculateAngle(card &theCard){
			
			float midPointX = 0.5 * theCard.c[ theCard.pta ].x + 0.5 * theCard.c[ theCard.ptb ].x; 
			float midPointY = 0.5 * theCard.c[ theCard.pta ].y + 0.5 * theCard.c[ theCard.ptb ].y; 
				
			float dx = midPointX - theCard.center.x;
			float dy = midPointY - theCard.center.y;
				
			float angle = atan2(dy, dx);
			
			theCard.angle = angle;
			
			//printf("angle is %f \n", angle);			
		}		
		

		
		//the heart of the beast - where we process 
		//---------------------------		
		int processFrame(int threshold, int minSize, int maxSize, int saveToBank = 0){
			
						
			///////////////////////////////////////////////////////////
			// Part 1 - get the video data
			///////////////////////////////////////////////////////////
			
			//pointer to our incoming video pixels			
			unsigned char * pixCam;
			
			//either grab pixels from video or grab from camera
			if(which == 1){
				VP.idleMovie();	
				pixCam 	= VP.getPixels();
		    }
		    else 
			if(which == 2){
				VG.grabFrame();
				pixCam 	= VG.getPixels();		
			}else{
				return;				
			}
	
	
			///////////////////////////////////////////////////////////
			// Part 2 - load into openCV and make greyscale
			///////////////////////////////////////////////////////////
			VideoFrame.setFromPixels(pixCam, W, H);
			GreyFrame.setFromColorImage(VideoFrame);
			WarpedFrame.warpIntoMe(GreyFrame, QUAD.getScaledQuadPoints(W,H), warpDst);
			
			
			///////////////////////////////////////////////////////////
			// Part 3 - do background subtraction
			///////////////////////////////////////////////////////////
			PresenceFrame.absDiff(WarpedFrame, BackgroundFrame);
			
			//PresenceFrame = WarpedFrame;
			//PresenceFrame -= BackgroundFrame;
			
			PresenceFrame.threshold(threshold);
			//PresenceFrame.erode_3x3();
		
			///////////////////////////////////////////////////////////
			// Part 4 - find the largest blobs of possible candidates 
			////////////////////////////////////////////////////////////
			numBlobs = Contour.findContours(PresenceFrame, minSize, maxSize, 50, false);  
			
			///////////////////////////////////////////////////////////
			// Part 5 - do shit with the data
			////////////////////////////////////////////////////////////
			blobTrack.identifyBlobs(Contour.blobs, numBlobs, 20);
			
			
			int k = 0;
			
			for(int i = 0; i < numBlobs; i++){
				 ofVec2f * tmpPts = contAna.smoothContour(Contour.blobs[i], 0.1);
				 
				 if(Contour.blobs[i].area < 320){
				 	
				 	continue;
				 }
				 
				 if(k < NUM_CARDS){
				 				 
				 	int numFound = CORNER[k].findCorners(tmpPts, Contour.blobs[i].nPts, 6);
				 				 
				 	if(numFound == 4 &&  blobTrack.getBlobActivity(i) < 0.8){
				 		cards[i].c[0] = tmpPts[ CORNER[k].getTipIndex(0) ];
				 		cards[i].c[1] = tmpPts[ CORNER[k].getTipIndex(1) ];
				 		cards[i].c[2] = tmpPts[ CORNER[k].getTipIndex(2) ];
				 		cards[i].c[3] = tmpPts[ CORNER[k].getTipIndex(3) ];
				 		cards[i].good = true;
				 						 	
				 		identifyShortestSide(cards[i]);
				 		calculateCenter(cards[i]);
				 		calculateAngle(cards[i]);
				 		
						int sx = Contour.blobs[i].boundingRect.x;
						int sy = Contour.blobs[i].boundingRect.y;
						int sw = Contour.blobs[i].boundingRect.width;
						int sh = Contour.blobs[i].boundingRect.height;
						
						unsigned char * pixels = PresenceFrame.getPixels();
						int whiteCount = 0;
						int blackCount = 0;
						
						float xArr[4] = {cards[i].c[0].x, cards[i].c[1].x, cards[i].c[2].x, cards[i].c[3].x};
						float yArr[4] = {cards[i].c[0].y, cards[i].c[1].y, cards[i].c[2].y, cards[i].c[3].y};
						
						ofPoint2f tmpQuad[4];
						ofPoint2f tmpDst[4];
						
						tmpDst[0].x = 0;
						tmpDst[0].y = 0;
						tmpDst[1].x = 64;
						tmpDst[1].y = 0;
						tmpDst[2].x = 64;
						tmpDst[2].y = 80;
						tmpDst[3].x = 0;
						tmpDst[3].y = 80;
						
						int pta = cards[i].pta;
						int ptb = cards[i].ptb;
						
						tmpQuad[0].x = cards[i].c[pta].x;
						tmpQuad[0].y = cards[i].c[pta].y;
						
						tmpQuad[1].x = cards[i].c[ptb].x;
						tmpQuad[1].y = cards[i].c[ptb].y;
						
						int ptc = -1;
						int ptd = -1;
						
						for(int j = 0; j < 4; j++){
							if(j != pta && j != ptb && ptc < 0)ptc = j;
							else if(j != pta && j != ptb && j != ptc && ptd < 0)ptd = j;
						}
						
						//printf("+PTA is %i PTB is %i PTC is %i PTD is %i\n", pta, ptb, ptc, ptd);

						float distPtbToPtc  =  sqrt( pow(cards[i].c[ptb].x - cards[i].c[ptc].x, 2) + pow(cards[i].c[ptb].y - cards[i].c[ptc].y, 2) );	
						float distPtbToPtd  =  sqrt( pow(cards[i].c[ptb].x - cards[i].c[ptd].x, 2) + pow(cards[i].c[ptb].y - cards[i].c[ptd].y, 2) );	
						
						
						if( distPtbToPtd < distPtbToPtc ){
							int tmp = ptd;
							ptd = ptc;
							ptc = tmp;
						}
						
						//printf("/PTA is %i PTB is %i PTC is %i PTD is %i\n", pta, ptb, ptc, ptd);
						
						cards[i].ptc = ptc;
						cards[i].ptd = ptd;
						
						tmpQuad[2].x = cards[i].c[ptc].x;
						tmpQuad[2].y = cards[i].c[ptc].y;
						
						tmpQuad[3].x = cards[i].c[ptd].x;
						tmpQuad[3].y = cards[i].c[ptd].y; 
						

						CardWarpFrame.warpIntoMe(WarpedFrame, tmpQuad, tmpDst);
						
						unsigned char * ptr = CardWarpFrame.getPixels();

						int smallest;
						int smallestID;
							
						int srcX = 0;
						int srcY = 0;
						
						int darkest = 255;
						int brightest = 0;
					
					
						unsigned char pix;
						
						for(int y = 0; y < 80; y++){
							for(int x = 0; x < 64; x++){
								pix = ptr[srcX + srcY*W];
								
								if(pix > brightest) brightest = pix;
								if(pix < darkest) darkest = pix;
								 	
								srcX++;
							}
							srcX = 0;
							srcY ++;
						}
						
						srcX = 0;
						srcY = 0;
						
						float pct = 0.0;
						float range = (float)(brightest - darkest);
						
						if(range > 0){
							for(int y = 0; y < 80; y++){
								for(int x = 0; x < 64; x++){
									pct = 255.0  *  ( (float)(ptr[srcX + srcY*W] - darkest)/range);
									if(pct > 128) singleCard[x + y*64] = 0;
									else singleCard[x + y*64] = 255;									
									srcX++;
								}
								srcX = 0;
								srcY ++;
							}
						}
						HuFrame.setFromPixels(singleCard, 64, 80);

						cvMoments(HuFrame.cvImage, &Moments);
						cvGetHuMoments(&Moments, &HuMoments);
						
						cards[i].hu = HuMoments.hu1*10000.0;
						lastVal = cards[i].hu;
							
				 		
				 		int closestMatch = 0;
				 		float smallestAmnt = 9999999;
				 		
						for(int p = 0; p < 10; p++){
												
							float diff = abs(cardVal[p] - cards[i].hu);
							
							if(diff < smallestAmnt){
								smallestAmnt = diff;
								cards[i].whichCard = p;
							}
						}

				 		
				 	}else{
				 		cards[i].whichCard = -1;
				 		cards[i].good = false;
				 		//printf("found %i points\n", numFound);
				 	}
				 }
				 
				 
				 k++;
			}
			
 			return numBlobs;			
		} 
	
		//----------------------------------------------------
		void calculateContourNormals(int contourStep){
			
			numPoints = 0;
			
			ofVec2f prev, current, perp;
			
			int numCompute = MIN(numBlobs, MAX_CONTOURS);
			
			for(int i = 0; i  < numCompute; i++){
				
				
				int contourLength = Contour.blobs[i].nPts;
				
				int step = contourStep;
				
				for(int j = 1; j < contourLength-1; j++){			
					
					prev.set(Contour.blobs[i].pts[j-1].x, Contour.blobs[i].pts[j-1].y);
					current.set(Contour.blobs[i].pts[j].x, Contour.blobs[i].pts[j].y);
					
					current = prev * 0.9  + current * 0.1;
					
					Contour.blobs[i].pts[j].x = current.x;
					Contour.blobs[i].pts[j].y = current.y;
					
					if( j % step == 0){
						perp = (current - prev);
						perp.set(-perp.y, perp.x);
						perp.normalize();
						contourList[numPoints].set(current.x/(float)W, current.y/(float)H, perp.x, perp.y);
						numPoints++;
					}
				}
			}
		}
		
		//---------------------------
		void storeCardVal(int which){
			cardVal[which] = lastVal;
		}
		
		//---------------------------
		ofVec4f * getContourPoints(){
			return 	contourList;
		}
		
		//--------------------------
		int getNumContourPoints(){
			return numPoints;
		}
		
		void saveCardBank(string path){
			tmp.setFromPixels( cardBank, W, H, OF_IMAGE_GRAYSCALE);
			tmp.saveImage(path);
		}
		
		//--------------------------
		void resetBlobID(){
			blobTrack.resetCounter();
		}
		
		//--------------------------
		void toggleVideoPlayback(){
			if(VP.getSpeed() == 0.0){
				VP.setSpeed(1.0);
			}else{
				VP.setSpeed(0.0);
			}
		}
		
		
		//---------------------------				
		void draw(float x, float y){
			
			//lets draw our openCV frames
			//with a nice border around them
			ofNoFill();
			ofSetColor(0xFFFFFF);	
			GreyFrame.draw(x, y, 320, 240);
			QUAD.draw(x, y, W, H);
			
			ofSetColor(0xFFFFFF);
			WarpedFrame.draw(x+320,y, 320, 240);
			
			CardWarpFrame.draw(x+320, y+240, 320, 240);
			BankFrame.draw(x, y+240, 320, 240);
						
			glPushMatrix();
				glTranslatef(320,0,0);
				Contour.draw();
			glPopMatrix();
			
			ofRect(x+320,y,W,H);

			PresenceFrame.draw(x+640,y, 160, 120);
			ofRect(x+640,y, 160, 120);
			
			BackgroundFrame.draw(x+640,y+120, 160, 120);
			ofRect(x+640,y+120, 160, 120);
			
			ofFill();
			
			ofSetColor(0xFF00FF);
			ofRect(0, 479, 320, 241);
			ofSetColor(0xFFFFFF);
			//tmp.draw(0, 480, 320, 240);
			HuFrame.draw(0,480);
			
			WarpedFrame.draw(320, 380, 640, 480);
			
			
			for(int i = 0; i < numBlobs; i++){
				
				int id 			= blobTrack.getBlobID(i);
				int duration 	= blobTrack.getBlobDuration(i);
				float activity 	= blobTrack.getBlobActivity(i);
			
				ofSetColor(0xFF2244);
				drawText(ofToString(id) +" - "+ofToString(duration)+" - "+ofToString(activity), Contour.blobs[i].centroid.x + 320, Contour.blobs[i].boundingRect.y);
			}
			
			if(numPoints > 0){
				ofSetColor(0x00FF00);
			
				float tx;
				float ty;
			
				for(int i = 0; i < numPoints; i++){
					tx = 320 + x + contourList[i].x* (float)W;
					ty = y + contourList[i].y*(float)H;
					ofLine(tx, ty, tx+contourList[i].z*10, ty+contourList[i].w*10);
				}
			} 
			
			glPushMatrix();
			glTranslatef(320,0,0);
			
			if(numBlobs > 0){
				for(int i = 0; i < numBlobs; i++){
					if(i < NUM_CARDS && cards[i].good){
						
						float px = Contour.blobs[i].centroid.x;
						float py = Contour.blobs[i].centroid.y;
						
						ofSetColor(0xFFFF00);
						ofCircle(px, py, 3);
						
						ofDrawBitmapString("hu is "+ofToString(cards[i].hu), px, py - 20);
						
						if(cards[i].whichCard != -1){
							ofSetColor(0xFFFFFF);
							TYPE.drawString(ofToString(cards[i].whichCard), px * 2, 2 * py + 300);
						}
						
						for(int k = 0; k < 4; k++){
						
							//printf("pta is %i and ptb is %i \n", cards[i].pta, cards[i].ptb);
						
							if(k == cards[i].pta || k == cards[i].ptb){
								ofSetColor(0xFF0000);
								if(k == cards[i].pta)ofDrawBitmapString("pta", cards[i].c[k].x, cards[i].c[k].y-10);
								else ofDrawBitmapString("ptb", cards[i].c[k].x, cards[i].c[k].y-10);
							} 
							else{
								 ofSetColor(0x00FFFF);
								if(k == cards[i].ptc)ofDrawBitmapString("ptc", cards[i].c[k].x, cards[i].c[k].y-10);
								else ofDrawBitmapString("ptd", cards[i].c[k].x, cards[i].c[k].y-10);
							}
							
							ofCircle(cards[i].c[k].x, cards[i].c[k].y, 3);
						}
						
						ofSetColor(0xFF00FF);
						glPushMatrix();
							glTranslatef(cards[i].center.x, cards[i].center.y, 0);
							glRotatef( 270 + (180 * cards[i].angle )/PI, 0, 0, 1);
							ofLine(0,0,0,30);
						glPopMatrix();

						
					}
				}
			}
			glPopMatrix();
			
		}
		
		
		
		guiQuad QUAD;
		
		int treeCandidateID;
		int numBlobs;
		bool newData;
		
		blobTracker blobTrack;
		contourAnalyser contAna;
		
		ofVideoPlayer  	VP;
		ofVideoGrabber 	VG;
		
		ofCvColorImage  		VideoFrame;
		ofCvGrayscaleImage 		GreyFrame;
		ofCvGrayscaleImage 		BankFrame;
		ofCvGrayscaleImage 		PresenceFrame;
		ofCvGrayscaleImage 		WarpedFrame;						
		ofCvGrayscaleImage 		CardWarpFrame;						
		ofCvGrayscaleImage 		BackgroundFrame;		
		ofCvContourFinder		Contour;
			
		bool bCameraSetup, bVideoSetup;
		bool bSetup;	
		
		ofVec4f contourList[MAX_CONTOURS];
		cornerFinder CORNER[NUM_CARDS];
		card cards[NUM_CARDS];

		ofCvGrayscaleImage		HuFrame;
		ofCvContourFinder HuContour;
		
		CvMoments Moments;
		CvHuMoments HuMoments;
		
		float cardVal[10];
		
		unsigned char cardBank[64*5*80*3];
		unsigned char singleCard[64*80];
		
		int numPoints;
				
		float lastVal;
		
		ofPoint2f 			warpSrc[4];
		ofPoint2f 			warpDst[4];
		
		ofImage tmp;

		
		int W;
		int H;
		
		int which;
		
		ofTrueTypeFont TYPE;

};



#endif