#ifndef _VIDEO_PROCESSING_H
#define _VIDEO_PROCESSING_H

#include "ofMain.h"
#include "ofAddons.h"
#include "guiQuad.h"

#include "baseGui.h"
#include "tipFinder.h"

#define NUM_BRANCHES 512


typedef struct{
	ofVec2f pos;
	ofVec2f vec;
	int	length;
}norm;

typedef struct{
	ofVec2f pos;
	int	which;
}branch;

static int block = 64;
static int partPos = 0;

//our other app objects
class videoProcessing : public baseGui{

	public:
	
		//---------------------------
		videoProcessing(){
		
			bCameraSetup 	= false;
			bVideoSetup  	= false;

			//we do all our tracking at W 240 - regardless 
			//of camera size - larger cameras get scaled down to
			//these dimensions - otherwise 'shit would be slow'
			pre				= new unsigned char[W * 240 * 3];
			
			lastActionTime  = 0;
			drawNormals		= false;
			normalCount		= 0;
			
			for(int i = 0; i < 1024; i++){
				bluredPoints[i].pos.x = 0;
				bluredPoints[i].pos.y = 0;
				tmpPoints[i].set(0,0);
			}
			
		}
		
		
		//---------------------------------------------
		void makeNormals(){
		
			ofVec2f horz(1,0);
			normalCount = 0;
			branchCount = 0;
				
			for (int j = 1; j < Contour.blobs[0].nPts-1; j++){
				
				 bluredPoints[j].which = -1;
				
				if(normalCount >= NUM_BRANCHES) break;
	
				float dx = bluredPoints[j+1].pos.x - bluredPoints[j-1].pos.x;
				float dy = bluredPoints[j+1].pos.y - bluredPoints[j-1].pos.y;

				ofVec2f dist(dx, dy);
				ofVec2f normal = dist.norm_value();
				
				//lets make a proper normal (perp to the tangent)
				normal.set(-normal.y, normal.x);
				
				float angle = normal.angle(horz);
				
				//then normal sould be facing up
				if(angle > 0 && angle < PI*2){
					branchNorms[normalCount].pos.set( bluredPoints[j].pos.x, bluredPoints[j].pos.y);
					
					if(normalCount%(1+rand()%59)==0){	
						branchNorms[normalCount].vec = normal * (float)(10 + rand()%20);						
					}else{
					 	branchNorms[normalCount].vec = normal * 5.0;
					}
					normalCount++;
				}
			}
			
			
			ofVec2f * ptr = &tmpPoints[partPos];
			
			ofVec2f centroid;
			
			centerY = Contour.blobs[0].boundingRect.y +  Contour.blobs[0].boundingRect.height  * 0.25;
			
			centroid.set(Contour.blobs[0].centroid.x, centerY);


			Tip.findTips(tmpPoints, Contour.blobs[0].nPts, centroid, 28);
			
			int count = 0;
			int result = 0;
			
			float rightMost = -999999;
			float leftMost  =  999999;
			
			int lID = -1;
			int rID = -1;
			
			
			printf("centerY is %f contour.y is %f\n", centerY, centroid.y);
			
			while( result != -1 ){
				
				result = Tip.getTipIndex(count);
				if(result == -1)break;
				
				//printf("%f \n", tmpPoints[result].x - centroid.x);
				
				if(leftMost > tmpPoints[result].x){
					 leftMost = tmpPoints[result].x;
					 lID = result;
					 
					 leftHand.set(tmpPoints[result].x - centroid.x, tmpPoints[result].y - centerY);
					 leftHand.normalize();
					 printf("SHIT\n");
				}
				
				if(rightMost < tmpPoints[result].x){
				 	rightMost = tmpPoints[result].x;
				 	
				 	rightHand.set(tmpPoints[result].x - centroid.x, tmpPoints[result].y - centerY);
					rightHand.normalize();
				 	
					 rID = result;
					 printf("SHIT2\n");
				}
				
				count++;
			}
			
			
			//printf("num is %i \n", normalCount);
			
		}
		
		
		//---------------------------
		void increaseSection(){
			 partPos += 64;
		}
		
		//---------------------------
		ofVec2f getLeftVector(){
			return leftHand;
		}
		
		//---------------------------
		ofVec2f getRightVector(){
			return rightHand;
		}
		
		//---------------------------				
		void draw(float x, float y){
			
			//lets draw our openCV frames
			//with a nice border around them
			ofNoFill();
			ofSetColor(0xFFFFFF);	
			GreyFrame.draw(0, 0, W, 240);
			ofRect(0,0,W,240);
			PresenceFrame.draw(320,0, W, 240);
			ofRect(320,0,W,240);
			BackgroundFrame.draw(640,0, W, 240);
			ofRect(640,0,W,240);			
			ofFill();
			
			Tip.draw(10, 700, 500, 300);
			
			//lets draw the contours to show our blobs!
			glPushMatrix();
				glTranslatef(W, 0, 0);
				ofSetColor(0xFF00FF);
				Contour.draw();	
			glPopMatrix();
				
				
				if(Contour.nBlobs > 0){
					
					ofEnableAlphaBlending();						

					glPushMatrix();
						glTranslatef(0,0,0);
						glColor4f(1.0, 1.0, 0.0, 0.5);
						
						ofFill();
						ofBeginShape();
							ofSetPolyMode(OF_POLY_WINDING_ODD);

							
							for (int j = 1; j < Contour.blobs[0].nPts; j++){
								
								ofVertex(bluredPoints[j].pos.x, bluredPoints[j].pos.y);
								/*
								if( drawNormals && bluredPoints[j].which != -1 ){
									
									printf("HERE\n");
									for(int i = 0; i < 8; i++){
										ofVertex(branches[bluredPoints[j].which][i].x, branches[bluredPoints[j].which][i].y);
									}
								}*/
								
							}
						
						ofEndShape();
						
						
						
						ofSetColor(30, 190, 255);
						for (int j = 0; j < Contour.blobs[0].nPts; j++){
							if(Tip.highest[j]){
								ofCircle(bluredPoints[j].pos.x, bluredPoints[j].pos.y, 2);
								ofLine(bluredPoints[j].pos.x, bluredPoints[j].pos.y, Contour.blobs[0].centroid.x, centerY);
							}
						}
						
						/*
						glColor4f(0.0, 1.0, 0.0, 0.9);
						if(drawNormals){
							for (int i = 0; i < normalCount; i++){
								ofLine(branchNorms[i].pos.x, branchNorms[i].pos.y, 
								branchNorms[i].pos.x + branchNorms[i].vec.x, 
								branchNorms[i].pos.y + branchNorms[i].vec.y);
							}
						}*/
						
					glPopMatrix();
					
					
					
						
					ofDisableAlphaBlending();						
				}
				
			
			
			
			
		}


		//---------------------------		
		//---------------------------		
		
		ofVideoPlayer  	VP;
		ofVideoGrabber 	VG;
		
		ofCvColorImage  		VideoFrame;
		ofCvGrayscaleImage 		GreyFrame;
		ofCvGrayscaleImage 		PresenceFrame;
		ofCvGrayscaleImage 		BackgroundFrame;		
		ofCvContourFinder		Contour;
		
		tipFinder				Tip;
		
		branch 					bluredPoints[1024];
		norm					branchNorms[NUM_BRANCHES];
		ofVec2f					branches[NUM_BRANCHES][8];
		ofVec2f					tmpPoints[1024];
		
		ofVec2f					leftHand;
		ofVec2f					rightHand;
		
		unsigned char * pre;
		
		string sendStr;
		
		bool bCameraSetup, bVideoSetup;
		
		int normalCount;
		int branchCount;
		bool drawNormals;
		
		long lastActionTime;
		long elapsedTime;
		
		float centerY;
		
		int W;
		int H;

};

#endif