#ifndef _CV_MANAGER_H
#define _CV_MANAGER_H

#include "ofMain.h"

#define OF_ADDON_USING_OFXVECTORMATH
#define OF_ADDON_USING_OFXOPENCV

#include "ofAddons.h"

#include "blobTracker.h"
#include "contourAnalyser.h"
#include "cornerFinder.h"

#define MAX_CONTOURS 10000
#define NUM_CARDS 9


typedef struct{
	ofxVec2f c[4];
	bool good;
	float pixelRatio;
	int whichCard;
	int pta;
	int ptb;
	int ptc;
	int ptd;
	float angle;
	float hu;
	ofxVec2f center;
}card;



// baseGui ????
class cvManager {
	
	public:
	cvManager();
		
		//changing cameras or switching from/to the camera mode
		//requires the app to be restarted - mabe we can change this?
		//---------------------------		
	void setupCamera(int deviceNumber, int width, int height);
	
	void setupVideo(string videoPath);
	
	//good for adjusting the color balance, brightness etc
		//---------------------------		
	void openCameraSettings();
	
		
		//---------------------------		
	void useCamera();
		
		//---------------------------		
	void useVideo();
	
		
		//set background 
		//---------------------------
	void captureBackgroundFrame();
		
		
		//do all our openCV allocation
		//---------------------------		
	void setupCV();
	
		
		
		//-----------------------------
	void identifyShortestSide(card &theCard);
		
		//-----------------------------
	void calculateCenter(card &theCard);
		
		//-----------------------------
	void calculateAngle(card &theCard);		

		
		//the heart of the beast - where we process 
		//---------------------------		
	int processFrame(int threshold, int minSize, int maxSize, int saveToBank = 0);
	
		//----------------------------------------------------
	void calculateContourNormals(int contourStep);
		
		//---------------------------
	void storeCardVal(int which);
		
		//---------------------------
	ofxVec4f * getContourPoints();		
		//--------------------------
	int getNumContourPoints();
	
	void saveCardBank(string path);
		
		//--------------------------
	void resetBlobID();
		
		//--------------------------
	void toggleVideoPlayback();
		
		
		//---------------------------				
	void draw(float x, float y);
		
		
		
		//guiQuad QUAD; // what to do ?
		
		int treeCandidateID;
		int numBlobs;
		bool newData;
		
		blobTracker blobTrack;
		contourAnalyser contAna;
		
		ofVideoPlayer  	VP;
		ofVideoGrabber 	VG;
		
		ofxCvColorImage  		VideoFrame;
		ofxCvGrayscaleImage 		GreyFrame;
		ofxCvGrayscaleImage 		BankFrame;
		ofxCvGrayscaleImage 		PresenceFrame;
		ofxCvGrayscaleImage 		WarpedFrame;						
		ofxCvGrayscaleImage 		CardWarpFrame;						
		ofxCvGrayscaleImage 		BackgroundFrame;		
		ofxCvContourFinder		Contour;
			
		bool bCameraSetup, bVideoSetup;
		bool bSetup;	
		
		ofxVec4f contourList[MAX_CONTOURS];
		cornerFinder CORNER[NUM_CARDS];
		card cards[NUM_CARDS];

		ofxCvGrayscaleImage		HuFrame;
		ofxCvContourFinder HuContour;
		
		CvMoments Moments;
		CvHuMoments HuMoments;
		
		float cardVal[10];
		
		unsigned char cardBank[64*5*80*3];
		unsigned char singleCard[64*80];
		
		int numPoints;
				
		float lastVal;
		
		ofxPoint2f 			warpSrc[4];
		ofxPoint2f 			warpDst[4];
		
		ofImage tmp;

		
		int W;
		int H;
		
		int which;
		
		ofTrueTypeFont TYPE;

};



#endif